/* sorgente: VettConc.c */

/* Programma che acquisisce due vettori di interi (sorgenti) la
** cui dimensione e nota a run-time, per concatenarli in un
** terzo vettore (destinazione) */

/* inclusione del file di intestazione della libreria standard
** che contiene definizioni di macro, costanti e dichiarazioni
** di funzioni e tipi funzionali alle varie operazioni di I/O */
#include <stdio.h>

/* inclusione del file di intestazione della libreria standard
** che contiene definizioni di macro, costanti e dichiarazioni
** di funzioni e tipi di interesse generale */
#include <stdlib.h>

/* inclusione del file di intestazione della libreria standard
** che contiene definizioni di macro, costanti e dichiarazioni
** di funzioni e tipi per la gestione delle stringhe */
#include <string.h>

/*
** funzione per l'acquisizione di un vettore di
** interi
*/
void AcqVettInt(int *Vett, unsigned int dim)
  /* Parametri formali
  **    Vett:      indirizzo della prima locazione assegnata
  **               al vettore
  **    dim:       dimensione del vettore */
  {
  /* definizione della variabile per la scansione del vettore */
  int pos;
  
  /* scansione del vettore e acquisizione del suo contenuto */
  for (pos = 0; pos < dim; pos++)
    {
    printf("\nVett[%d]? ", pos);
    scanf("%d", Vett+pos);
    };
  }
  
/*
** funzione per la restituzione del contenuto di un vettore
** di interi
*/
void ResVettInt(int *Vett, unsigned int dim)
  /* Parametri formali
  **    Vett:      indirizzo della prima locazione assegnata
  **               al vettore
  **    dim:       dimensione del vettore */
  {  
  /* definizione della variabile per la scansione del vettore */
  int pos;
  
  /* scansione del vettore e restituzione del suo contenuto */
  for (pos = 0; pos < dim; pos++)
    printf("\nVett[%d]: %d", pos, *(Vett+pos));
  }

/*
** funzione per l'allocazione di un buffer la cui
** dimensione e' espressa in numero di interi
*/
int * AllBuffInt(unsigned int dim)
  /* Parametri formali
  **    dim: dimensione del buffer in numero di interi */
  {
  /* definizione della variabile per lindirizzo
  ** iniziale del buffer */
  int *ptr;
  
  /* allocazione della memoria per il buffer */
  ptr = (int *) malloc(dim * sizeof(int));

  /* se lallocazione e andata a buon fine restituisce
  ** lindirizzo iniziale del buffer; NULL altrimenti */

  return(ptr);
  }
 
/* funzione che concatena due vettori di interi in un terzo
** vettore */
int *ConcVettInt(int *sorg1, unsigned int dim1, int *sorg2, unsigned int dim2)

  {
  /* definizione del "nome" del vettore risultante */
  int *result;
  
  /* allocazione della memoria per il vettore risultante */
  if ((result = AllBuffInt(dim1+dim2)) == NULL)
    {
    /* se l'allocazione non ha esito positivo restituisce NULL */
    return(result);
    }
    
  /* copia del contenuto del I vettore sorgente
  ** in testa al vettore destinazione */
  memcpy((void *) result, (void *) sorg1, dim1*sizeof(int));
  
  /* copia del contenuto del II vettore sorgente in coda al I 
  ** nel vettore destinazione */
  memcpy((void *) (result+dim1), (void *) sorg2, dim2*sizeof(int));
  
  /* restituisce l'indirizzo iniziale del vettore risultante */
  return(result);
  }
  
/* Chiamante */
int main()
  {
  /* definizione delle variabili per gli indirizzi dei tre vettori */
  int *src1, *src2, *trg;
  
  /* definizione delle variabili per la dimensione dei due vettori
  ** sorgente */
  unsigned int dim_src1, dim_src2;
  
  /* acquisizione della dimensione del I vettore sorgente */
  printf("\nDimensione del I vettore? ");
  scanf("%u", &dim_src1);
  
  /* allocazione della memoria per il I vettore sorgente */
  if ((src1 = AllBuffInt(dim_src1)) == NULL)
    {
    /* se l'allocazione fallisce, termina */
    printf("\nEsito dell'allocazione del I sorgente negativo");
    return(0);
    };
  
  /* acquisizione della dimensione del II vettore sorgente */
  printf("\nDimensione del II vettore? ");
  scanf("%u", &dim_src2);
  
  /* allocazione della memoria per il II vettore sorgente */
  if ((src2 = AllBuffInt(dim_src2)) == NULL)
    {
    /* se l'allocazione fallisce, rilascia la memoria per il
    ** I vettore e termina */
    printf("\nEsito dell'allocazione del II sorgente negativo");
    free(src1);
    
    return(0);
    };
    
  /* acquisizione del contenuto dei due vettori sorgente */
  printf("\nAcquisizione del I vettore");
  AcqVettInt(src1, dim_src1);

  /* printf("\nAcquisizione del II vettore"); */
  AcqVettInt(src2, dim_src2);
  
  /* concatenazione dei due vettori sorgente nel vettore destinazione:
  ** se lesito  negativo rilascia la memoria allocata per i
  ** sorgenti e termina */
  if ((trg = ConcVettInt(src1, dim_src1, src2, dim_src2)) == NULL)
    {
    printf("\nEsito dell'allocazione del target negativo");
    free(src1);
    free(src2);
    
    return(0);
    };
  
  /* restituzione del contenuto del vettore destinazione */
  printf("\nContenuto del vettore risultante:");
  ResVettInt(trg, dim_src1+dim_src2);
  
  /* recupero della memoria allocata per i tre vettori */
  free(src1);
  free(src2);
  free(trg);
  
  return(1);
  }
